@@ -259,7 +259,7 @@ ps_root_doioctl(unsigned long req, void *data, size_t len)
 		return -1;
 	}
 
-	s = socket(PF_INET, SOCK_DGRAM, 0);
+	s = xsocket(PF_INET, SOCK_DGRAM, 0);
 	if (s != -1)
 #ifdef IOCTL_REQUEST_TYPE
 	{
@@ -685,17 +685,6 @@ ps_root_startcb(struct ps_process *psp)
 
 	if (if_opensockets(ctx) == -1)
 		logerr("%s: if_opensockets", __func__);
-#ifdef BSD
-	else {
-		/* We only want to write to this socket, so set
-		 * a small as possible buffer size. */
-		socklen_t smallbuf = 1;
-
-		if (setsockopt(ctx->link_fd, SOL_SOCKET, SO_RCVBUF,
-		    &smallbuf, (socklen_t)sizeof(smallbuf)) == -1)
-			logerr("%s: setsockopt(SO_RCVBUF)", __func__);
-	}
-#endif
 
 	/* Open network sockets for sending.
 	 * This is a small bit wasteful for non sandboxed OS's
@@ -899,7 +888,7 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 		return -1;
 #endif
 
-	if (socketpair(AF_UNIX, SOCK_SEQPACKET | SOCK_CXNB, 0, datafd) == -1)
+	if (xsocketpair(AF_UNIX, SOCK_SEQPACKET | SOCK_CXNB, 0, datafd) == -1)
 		return -1;
 	if (ps_setbuf_fdpair(datafd) == -1)
 		return -1;
@@ -937,6 +926,32 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 	return pid;
 }
 
+void
+ps_root_close(struct dhcpcd_ctx *ctx)
+{
+
+	if_closesockets(ctx);
+
+#ifdef INET
+	if (ctx->udp_wfd != -1) {
+		close(ctx->udp_wfd);
+		ctx->udp_wfd = -1;
+	}
+#endif
+#ifdef INET6
+	if (ctx->nd_fd != -1) {
+		close(ctx->nd_fd);
+		ctx->nd_fd = -1;
+	}
+#endif
+#ifdef DHCP6
+	if (ctx->dhcp6_wfd != -1) {
+		close(ctx->dhcp6_wfd);
+		ctx->dhcp6_wfd = -1;
+	}
+#endif
+}
+
 int
 ps_root_stop(struct dhcpcd_ctx *ctx)
 {
