@@ -45,7 +45,7 @@ ps_root_doioctl6(unsigned long req, void *data, size_t len)
 {
 	int s, err;
 
-	s = socket(PF_INET6, SOCK_DGRAM, 0);
+	s = xsocket(PF_INET6, SOCK_DGRAM, 0);
 	if (s != -1)
 		err = ioctl(s, req, data, len);
 	else
@@ -63,7 +63,7 @@ ps_root_doroute(void *data, size_t len)
 	int s;
 	ssize_t err;
 
-	s = socket(PF_ROUTE, SOCK_RAW, 0);
+	s = xsocket(PF_ROUTE, SOCK_RAW, 0);
 	if (s != -1)
 		err = write(s, data, len);
 	else
